#!/usr/bin/python3

import sys
import os
import argparse
import subprocess
import time
import managers.FileRestrictionManager as FileRestrictionManager


import managers.ProfileManager as ProfileManager
import managers.NetworkFilterManager as NetworkFilterManager
import managers.ApplicationManager as ApplicationManager

import Logger

CWD = os.path.dirname(os.path.abspath(__file__))


def is_service_active():
    return os.path.exists(ProfileManager.APPLIED_PROFILE_PATH)


def run_activator(params):
    process = subprocess.run(
        ["pkexec", CWD + "/" + os.path.basename(__file__)] + params, capture_output=True
    )

    Logger.log("Activator run with params: {}".format(params))

    return process


class ETAKisitActivator:
    def __init__(self, args):
        self.args = args
        print("args:", args)

        # Privileged run check
        if not FileRestrictionManager.check_user_privileged():
            sys.stderr.write("You are not privileged to run this script.\n")
            sys.exit(1)

    def run(self):
        print("== ETAKisitActivator STARTED ==")
        print("uid:", os.getuid())
        print("gid:", os.getgid())

        is_restrict = True if self.args.restrict else False
        is_unrestrict = True if self.args.unrestrict else False
        is_enable_websites = True if self.args.enable_websites_restriction else False
        is_disable_websites = True if self.args.disable_websites_restriction else False

        # Read Profiles
        self.read_profile()
        self.read_applied_profile()

        # Websites
        if is_enable_websites and is_service_active():
            self.set_network_filter(True, self.applied_profile)
        elif is_disable_websites or is_unrestrict:
            self.set_network_filter(False, self.applied_profile)

        # Applications
        if is_restrict:
            self.set_application_filter(True, self.profile)
        elif is_unrestrict:
            self.set_application_filter(False, self.applied_profile)

        # Save/Remove json file
        if is_restrict:
            self.save_applied_profile()
        elif is_unrestrict:
            self.remove_applied_profile()

        print("== ETAKisitActivator FINISHED ==")

    def read_profile(self):
        self.profile = ProfileManager.get_default().get_current_profile()

    def read_applied_profile(self):
        if is_service_active():
            self.applied_profile = ProfileManager.Profile(
                ProfileManager.get_default().load_json_from_file(
                    ProfileManager.APPLIED_PROFILE_PATH
                )
            )

    def save_applied_profile(self):
        profile_manager = ProfileManager.get_default()
        current_profile = profile_manager.get_current_profile()

        profile_manager.save_as_json_file(
            ProfileManager.APPLIED_PROFILE_PATH, current_profile
        )

    def remove_applied_profile(self):
        if os.path.isfile(ProfileManager.APPLIED_PROFILE_PATH):
            os.remove(ProfileManager.APPLIED_PROFILE_PATH)

    def set_application_filter(self, is_activate, profile):
        if is_activate:
            restriction_type = profile.get_application_restriction_type()

            if restriction_type == "allowlist":
                all_applications = ApplicationManager.get_all_applications()

                for app in all_applications:
                    if app.get_id() not in profile.get_application_allowlist():
                        ApplicationManager.restrict_application(app.get_id())
            elif restriction_type == "denylist":
                for app_id in profile.get_application_denylist():
                    ApplicationManager.restrict_application(app_id)
            else:
                # restriction_type = "none"
                pass

        else:
            restriction_type = profile.get_application_restriction_type()

            if restriction_type == "allowlist":
                all_applications = ApplicationManager.get_all_applications()
                for app in all_applications:
                    # Remove All app restrictions
                    ApplicationManager.unrestrict_application(app.get_id())
            elif restriction_type == "denylist":
                for app_id in profile.get_application_denylist():
                    ApplicationManager.unrestrict_application(app_id)
            else:
                # restriction_type = "none"
                pass

    def set_network_filter(self, is_activate, profile):
        profile_manager = ProfileManager.get_default()

        if is_activate:
            restriction_type = profile.get_website_restriction_type()

            if restriction_type == "none":
                return

            # browser + domain configs
            if restriction_type == "allowlist":
                website_list = profile.get_website_allowlist()

                NetworkFilterManager.set_domain_filter_list(
                    website_list, True, profile_manager.get_base_dns_server()
                )
            elif restriction_type == "denylist":
                website_list = profile.get_website_denylist()
                if len(website_list) == 0:
                    return

                NetworkFilterManager.set_domain_filter_list(
                    website_list, False, profile_manager.get_base_dns_server()
                )

            # resolvconf
            NetworkFilterManager.set_resolvconf_to_localhost()

            # smartdns-rs
            NetworkFilterManager.enable_smartdns_service()
            time.sleep(1)
            NetworkFilterManager.restart_smartdns_service()
        else:
            restriction_type = profile.get_website_restriction_type()

            if restriction_type == "none":
                return

            # browser + domain configs
            NetworkFilterManager.reset_domain_filter_list()

            # resolvconf
            NetworkFilterManager.reset_resolvconf_to_default()

            # smartdns-rs
            NetworkFilterManager.stop_smartdns_service()
            NetworkFilterManager.disable_smartdns_service()


if __name__ == "__main__":
    # Argument Parsing
    parser = argparse.ArgumentParser(
        description="Application & Domain restricting activator."
    )

    # Restrict Arguments
    parser.add_argument(
        "--restrict",
        action="store_true",
        help="Restrict applications and websites in current profile.",
    )
    parser.add_argument(
        "--enable-websites-restriction",
        action="store_true",
        help="Enable website restriction in applied profile.",
    )

    # Unrestrict Arguments
    parser.add_argument(
        "--unrestrict",
        action="store_true",
        help="Unrestrict applications and websites in applied profile",
    )
    parser.add_argument(
        "--disable-websites-restriction",
        action="store_true",
        help="Enable website restriction in applied profile.",
    )
    args = parser.parse_args()

    activator = ETAKisitActivator(args)
    activator.run()
