import ui_gtk3.PActionRow as PActionRow
from ui_gtk3.Dialogs import InputDialog
from managers.ProfileManager import ProfileManager
import ui_gtk3.Dialogs as Dialogs
import random
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, GLib  # noqa


class ProfileChooserWindow(Gtk.Window):
    def __init__(
        self, profile_manager: ProfileManager, parent_window, cmb_current_profile
    ):
        super().__init__(
            transient_for=parent_window,
            modal=True,
            window_position=Gtk.WindowPosition.CENTER_ON_PARENT,
        )

        self.profile_manager = profile_manager
        self.cmb_current_profile = cmb_current_profile

        self.setup_window()

        self.setup_ui()

        self.selected_profile = self.profile_manager.get_current_profile_name()

    # == SETUP ==
    def setup_window(self):
        self.set_default_size(400, 400)
        self.set_title("Ayar Listesi")
        self.connect("delete-event", self.on_window_delete)

    def setup_ui(self):
        box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            spacing=7,
            margin=14,
        )

        # New Profile button
        hbox1 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=7)
        btn_new = Gtk.Button.new_from_stock(Gtk.STOCK_ADD)
        btn_new.set_halign(Gtk.Align.END)
        btn_new.set_always_show_image(True)
        btn_new.connect("clicked", self.on_btn_new_clicked)
        hbox1.add(btn_new)

        box.add(hbox1)

        # Profiles ListBox
        self.listbox_profiles = Gtk.ListBox()

        scrolledwindow = Gtk.ScrolledWindow(
            max_content_width=386, min_content_height=240, vexpand=True
        )
        frame = Gtk.Frame()
        frame.add(scrolledwindow)
        scrolledwindow.add(self.listbox_profiles)
        box.add(frame)

        # Finish
        self.add(box)

        self.fill_profiles_group()

    # == FUNCTIONS ==
    def add_profile_entry_row(self, profile_name, is_editable=True):
        if is_editable:
            action_row = PActionRow.new(
                title=profile_name,
                on_deleted=self.on_btn_delete_row_clicked,
                on_edited=self.on_profile_name_changed,
                on_duplicated=self.on_profile_duplicate_clicked,
            )

            self.listbox_profiles.add(action_row)
        else:
            action_row = PActionRow.new(
                title=profile_name,
                on_duplicated=self.on_profile_duplicate_clicked,
            )

            self.listbox_profiles.add(action_row)

        self.listbox_profiles.show_all()

    def fill_profiles_group(self):
        for profile_name, profile in self.profile_manager.get_profile_list().items():
            if profile.get_is_default():
                self.add_profile_entry_row(profile_name, is_editable=False)
                break

        for profile_name, profile in self.profile_manager.get_profile_list().items():
            if not profile.get_is_default():
                self.add_profile_entry_row(profile_name)

    # == CALLBACKS ==
    def on_window_delete(self, win, event):
        # Don't delete window on close, just hide.
        self.hide()
        return True

    def on_btn_new_clicked(self, btn):
        input_dialog = InputDialog(
            self, "İsim giriniz", "", self.on_new_profile_entered, None, "Yeni Ayar"
        )
        input_dialog.show_all()

    def on_new_profile_entered(self, text):
        new_profile_name = text

        if self.profile_manager.has_profile_name(new_profile_name):
            Dialogs.info(
                "Hata: '{}' ismi zaten mevcut!".format(new_profile_name),
                Gtk.MessageType.ERROR,
            )

        else:
            # New profile created
            self.profile_manager.insert_default_profile(new_profile_name)

            # Update profiles in Gtk.ComboBoxText
            self.cmb_current_profile.append_text(new_profile_name)

            # Add Row
            self.add_profile_entry_row(new_profile_name)

            Dialogs.info("Yeni ayar oluşturuldu: '{}'".format(new_profile_name))

    def on_btn_delete_row_clicked(
        self, btn, action_row, user_data, _parent_listmodel, _is_model_allowlist
    ):
        # Checks
        if len(self.profile_manager.get_profile_list()) == 1:
            Dialogs.info("En az bir ayar olmak zorundadır.")
            return False

        profile_name = action_row.title
        current_profile = self.profile_manager.get_current_profile_name()

        if profile_name == current_profile:
            Dialogs.info("Etkin ayar silinemez.")
            return False

        # Ask user, are you sure?
        result = Dialogs.ok_cancel("'{}' silinecektir.".format(profile_name))
        if result != Gtk.ResponseType.OK:
            return False

        # Remove from ListBox
        self.listbox_profiles.remove(action_row)

        # Remove from Gtk.ComboBoxText
        iter = self.cmb_current_profile.get_model().get_iter_first()
        i = 0
        while iter is not None:
            row_text = self.cmb_current_profile.get_model().get_value(iter, 0)
            if row_text == profile_name:
                self.cmb_current_profile.remove(i)
                break

            i += 1
            iter = self.cmb_current_profile.get_model().iter_next(iter)

        # Remove from profiles.json
        self.profile_manager.remove_profile(profile_name)

    def on_profile_name_changed(self, entry, row, user_data):
        new_text = entry.get_text()

        if new_text == "":
            Dialogs.info("Ayar ismi boş olamaz!")
            return False

        old_text = row.title

        if self.profile_manager.has_profile_name(new_text):
            Dialogs.info(
                "Hata: '{}' zaten mevcut!".format(new_text),
                Gtk.MessageType.ERROR,
            )

            return False

        # Update profiles.json
        self.profile_manager.update_profile_name(old_text, new_text)

        # Update Gtk.ComboBoxText
        for modelrow in self.cmb_current_profile.get_model():
            if modelrow[0] == old_text:
                modelrow[0] = new_text
                break

        row.title = new_text

        # If current profile's name changed
        if self.selected_profile == old_text:
            self.selected_profile = new_text
            self.profile_manager.set_current_profile(self.selected_profile)

        Dialogs.info("Ayar ismi değiştirildi: '{}'".format(new_text))

    def on_profile_duplicate_clicked(self, btn, action_row, user_data):
        profile_name = action_row.title
        new_profile_name = "{}(1)".format(profile_name)

        # Generate random names until find an unused one
        while self.profile_manager.has_profile_name(new_profile_name):
            random_number = random.getrandbits(24)
            new_profile_name = "{}({})".format(profile_name, random_number)

        # New profile create
        self.profile_manager.duplicate_profile(profile_name, new_profile_name)

        # Update profiles in Gtk.ComboBoxText
        self.cmb_current_profile.append_text(new_profile_name)

        # Add Row
        self.add_profile_entry_row(new_profile_name)

        Dialogs.info("Yeni ayar oluşturuldu: '{}'".format(new_profile_name))
